@@ -30,19 +30,19 @@
 #	include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <glib.h>
-#include "egg-action-group.h"
+#include <gtk/gtk.h>
+#include "gtk24backports.h"
 #include "kz-window.h"
 #include "kz-mozembed.h"
 #include "kz-bookmark.h"
 
 G_BEGIN_DECLS
 
-extern EggActionGroupEntry kz_actions[];
-extern const gint          kz_actions_len;
+extern GtkActionEntry kz_actions[];
+extern const gint     kz_actions_len;
 
-EggActionGroup *kz_actions_create_group              (KzWindow      *kz,
-					              GtkAccelGroup *accels);
+GtkActionGroup *kz_actions_create_group              (KzWindow      *kz,
+						      GtkAccelGroup *accel_group);
 
 void            kz_actions_set_sensitive             (KzWindow      *kz,
 						      KzMozEmbed    *kzembed);
@@ -57,6 +57,16 @@ void            kz_actions_popup_bookmark_menu_modal (KzWindow      *kz,
 						      guint          button,
 						      guint          time);
 
+void            kz_actions_insert_smartbookmark      (KzWindow      *kz,
+						      KzBookmark    *bookmark,
+						      KzBookmark    *child,
+						      KzBookmark    *sibling);
+void            kz_actions_remove_smartbookmark      (KzWindow      *kz,
+						      KzBookmark    *bookmark,
+						      KzBookmark    *child);
+void            kz_actions_update_smartbookmarks     (KzWindow      *kz,
+						      KzBookmark    *bookmark);
+
 G_END_DECLS
 
 #endif /* __KZ_ACTIONS_H__ */
