@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-edit.h,v 1.5 2004/03/14 08:56:06 makeinu Exp $
+ *  $Id: kz-bookmark-edit.h,v 1.6 2004/07/15 13:19:53 ikezoe Exp $
  */
 
 #ifndef __KZ_BOOKMARK_EDIT_H__
@@ -38,6 +38,7 @@ G_BEGIN_DECLS
 typedef struct _KzBookmarkEdit		KzBookmarkEdit;
 typedef struct _KzBookmarkEditClass	KzBookmarkEditClass;
 typedef struct _KzBookmarkEditPriv	KzBookmarkEditPriv;
+typedef struct _KzSmartBookmarkEdit	KzSmartBookmarkEdit;
 
 struct _KzBookmarkEdit
 {
@@ -48,6 +49,9 @@ struct _KzBookmarkEdit
 	GtkWidget  *location_label, *location_entry;
 	GtkWidget  *memo_label, *memo_textview, *memo_scrwin;
 	GtkWidget  *interval_label, *interval_spin;
+	
+	KzSmartBookmarkEdit *smart;
+
 
 	gboolean    changing; /* FIXME: use private struct */
 
