@@ -18,15 +18,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark.c,v 1.36 2004/05/19 05:17:02 ikezoe Exp $
+ *  $Id: kz-bookmark.c,v 1.45 2004/07/18 09:10:13 ikezoe Exp $
  */
 
 #include "kz-bookmark.h"
 
 #include <string.h>
+#include <gtk/gtk.h>
 #include "gobject-utils.h"
 #include "intl.h"
-#include "egg-marshalers.h"
+#include "kz-marshalers.h"
+#include "utils.h"
+#include "eggregex.h"
 
 #include "kz-io.h"
 
@@ -36,7 +39,6 @@
 #include "kz-nsbookmark.h"
 #include "kz-lirs.h"
 #include "kz-hinadi.h"
-#include "kz-google.h"
 #include "kz-scrap-bookmark.h"
 
 enum {
@@ -63,10 +65,12 @@ enum {
 	PROP_DESCRIPTION,
 	PROP_LAST_MODIFIED,
 	PROP_INTERVAL,
+	PROP_SMART_LIST,
 	PROP_EDITABLE
 };
 
-#define ADD_BOOKMARK_FILE_TYPE(func) 					\
+
+#define ADD_BOOKMARK_FILE_TYPE(func)					\
 {									\
 	KzBookmarkFileType *type;					\
 	gint i;								\
@@ -99,9 +103,13 @@ static void kz_bookmark_set_editable      (KzBookmark *bookmark,
 
 static gboolean kz_bookmark_is_loading_all_children (KzBookmark *bookmark);
 
-static void cb_io_load_complete           (KzIO *io,
-					   GError *error,
-					   KzBookmark   *bookmark);
+static void     cb_io_load_complete           (KzIO *io,
+					       GError *error,
+					       KzBookmark   *bookmark);
+static gboolean idle_load_complete            (gpointer data);
+
+static void     free_smart_list               (GList *list);
+
 static KzBookmarkFileType *
             kz_bookmark_detect_file_type  (KzBookmark   *bookmark,
 					   gchar        *buf);
@@ -117,11 +125,14 @@ static GQuark location_quark          = 0;
 static GQuark file_type_quark         = 0;
 static GQuark description_quark       = 0;
 static GQuark last_mod_quark          = 0;
+static GQuark smart_list_quark        = 0;
 static GQuark parent_quark            = 0;
 static GQuark children_quark          = 0;
 static GQuark interval_quark          = 0;
 static GQuark timer_quark             = 0;
 static GQuark state_quark             = 0;
+static GQuark io_quark                = 0;
+static GQuark io_signal_id_quark      = 0;
 
 
 GType
@@ -158,7 +169,6 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 		ADD_BOOKMARK_FILE_TYPE(kz_rss_get_file_types);
 		ADD_BOOKMARK_FILE_TYPE(kz_lirs_get_file_types);
 		ADD_BOOKMARK_FILE_TYPE(kz_hinadi_get_file_types);
-		ADD_BOOKMARK_FILE_TYPE(kz_google_get_file_types);
 		ADD_BOOKMARK_FILE_TYPE(kz_xbel_get_file_types);
 		ADD_BOOKMARK_FILE_TYPE(kz_nsbookmark_get_file_types);
 		ADD_BOOKMARK_FILE_TYPE(kz_scrap_bookmark_get_file_types);
@@ -285,13 +295,22 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 			 TRUE,
 			 G_PARAM_READWRITE));
 
+	g_object_class_install_property(
+		object_class,
+		 PROP_SMART_LIST,
+		 g_param_spec_pointer(
+			 "smart-list",
+			 _("Smart List"),
+			 _("The List for the Smart Bookmark"),
+			 G_PARAM_READWRITE));
+
 	kz_bookmark_signals[INSERT_CHILD_SIGNAL]
 		= g_signal_new ("insert-child",
 				G_TYPE_FROM_CLASS (klass),
 				G_SIGNAL_RUN_LAST,
 				G_STRUCT_OFFSET (KzBookmarkClass, insert_child),
 				NULL, NULL,
-				_egg_marshal_VOID__OBJECT_OBJECT,
+				_kz_marshal_VOID__OBJECT_OBJECT,
 				G_TYPE_NONE, 2,
 				KZ_TYPE_BOOKMARK, KZ_TYPE_BOOKMARK);
 
@@ -311,7 +330,7 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 				G_SIGNAL_RUN_LAST,
 				G_STRUCT_OFFSET (KzBookmarkClass, move_child),
 				NULL, NULL,
-				_egg_marshal_VOID__OBJECT_OBJECT,
+				_kz_marshal_VOID__OBJECT_OBJECT,
 				G_TYPE_NONE, 2,
 				KZ_TYPE_BOOKMARK, KZ_TYPE_BOOKMARK);
 
@@ -397,6 +416,9 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 	interval_quark    = g_quark_from_string("KzBookmark::Interval");
 	timer_quark       = g_quark_from_string("KzBookmark::Timer");
 	state_quark       = g_quark_from_string("KzBookmark::State");
+	io_quark          = g_quark_from_string("KzBookmark::KzIO");
+	io_signal_id_quark= g_quark_from_string("KzBookmark::KzIOSignalID");
+	smart_list_quark  = g_quark_from_string("KzBookmark::SmartBookmarkList");
 }
 
 
@@ -412,7 +434,9 @@ static void
 kz_bookmark_dispose (GObject *object)
 {
 	KzBookmark *bookmark;
-	guint timer_id;
+	guint timer_id, io_id;
+	KzIO *io;
+	GList *smart_list;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(object));
 
@@ -426,6 +450,34 @@ kz_bookmark_dispose (GObject *object)
 	timer_id = 0;
 	g_object_set_qdata(object, timer_quark, GUINT_TO_POINTER(timer_id));
 
+	/* dispose KzIO object if exists */
+	/* FIXME! we need more robust way */
+	io = g_object_get_qdata(object, io_quark);
+	if (io)
+	{
+		kz_io_stop(io);
+
+	}
+
+	io_id = GPOINTER_TO_UINT(g_object_get_qdata(object, io_signal_id_quark));
+	if (io_id)
+	{
+		/* wait until disconnecting (i.e. loading stops  )*/
+		while (g_signal_handler_is_connected(G_OBJECT(io), io_id))
+			gtk_main_iteration();
+	}
+	g_object_set_qdata(object, io_quark, NULL);
+
+	io_id = 0;
+	g_object_set_qdata(object, io_signal_id_quark,
+			   GUINT_TO_POINTER(io_id));
+
+	/* smart property */
+	
+	smart_list = g_object_get_qdata(object, smart_list_quark);
+	free_smart_list(smart_list);
+	g_object_set_qdata(object, smart_list_quark, NULL);
+	
 	if (G_OBJECT_CLASS (parent_class)->dispose)
 		G_OBJECT_CLASS (parent_class)->dispose(object);
 }
@@ -438,6 +490,30 @@ kz_bookmark_dispose (GObject *object)
 }
 
 
+static void
+free_smart_list (GList *list)
+{
+	GList *node;
+
+	if (!list) return;
+
+	for (node = list; node; node = g_list_next(node))
+	{
+		KzBookmarkSmartProperty *prop = node->data;
+		if (!prop) continue;
+	
+		if (prop->regex)
+			g_free(prop->regex);
+		if (prop->uri)
+			g_free(prop->uri);
+		if (prop->encode)
+			g_free(prop->encode);
+		g_free(prop);
+	}
+	g_list_free(list);
+	list = NULL;
+}
+
 static void
 kz_bookmark_set_property (GObject *object,
 			  guint prop_id,
@@ -536,6 +612,11 @@ kz_bookmark_set_property (GObject *object,
 		else
 			bookmark->flags &= ~KZ_BOOKMARK_EDITABLE_FLAG;
 		break;
+	case PROP_SMART_LIST:
+		g_return_if_fail(kz_bookmark_is_smart(bookmark));
+		g_object_set_qdata(object, smart_list_quark, 
+				   g_value_get_pointer(value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		break;
@@ -601,6 +682,10 @@ kz_bookmark_get_property (GObject *object,
 		else
 			g_value_set_boolean(value, FALSE);
 		break;
+	case PROP_SMART_LIST:
+		g_value_set_pointer(value, 
+				    g_object_get_qdata(object, smart_list_quark));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		break;
@@ -972,6 +1057,216 @@ kz_bookmark_set_folded (KzBookmark  *bookmark, gboolean folded)
 }
 
 
+gboolean
+kz_bookmark_is_smart (KzBookmark *bookmark)
+{
+	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
+	return bookmark->flags & KZ_BOOKMARK_SMART_FLAG ? TRUE : FALSE;
+}
+
+void
+kz_bookmark_append_smart_property (KzBookmark *bookmark,
+				   const gchar *regex,
+				   const gchar *uri,
+				   const gchar *encode,
+				   gboolean urlencode)
+{
+	GList *smart_list;
+	KzBookmarkSmartProperty *prop;
+
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+
+	bookmark->flags |= KZ_BOOKMARK_SMART_FLAG;
+
+	prop = g_new0(KzBookmarkSmartProperty, 1);
+	if (regex)
+		prop->regex = g_strdup(regex);
+	if (uri)
+		prop->uri   = g_strdup(uri);
+	if (encode)
+		prop->encode = g_strdup(encode);
+	prop->urlencode = urlencode;
+
+	smart_list = g_object_get_qdata(G_OBJECT(bookmark),
+					smart_list_quark);
+	smart_list = g_list_append(smart_list, prop);
+
+	g_object_set(G_OBJECT(bookmark), "smart-list", smart_list, NULL);
+}
+
+void
+kz_bookmark_remove_smart_property (KzBookmark *bookmark)
+{
+}
+
+#define MAX_GROUPSTRING 16
+gchar *
+kz_bookmark_get_smart_uri (KzBookmark *bookmark, const gchar *text)
+{
+	gchar *smart_uri;
+	GList *smart_list, *node;
+	KzBookmarkSmartProperty *match = NULL;
+	EggRegex *egg_regex;
+	GError *error = NULL;
+
+	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);
+
+	if (!(bookmark->flags & KZ_BOOKMARK_SMART_FLAG))
+		return NULL;
+
+	if (!text)
+	{
+		smart_uri = g_strdup(kz_bookmark_get_link(bookmark));
+		return smart_uri;
+	}
+
+	smart_list = g_object_get_qdata(G_OBJECT(bookmark),
+					smart_list_quark);
+
+	for (node = smart_list; node; node = g_list_next(node))
+	{
+		gchar *regex;
+		KzBookmarkSmartProperty *prop;
+		
+		prop = node->data;	
+		regex = prop->regex;
+
+		egg_regex = egg_regex_new(regex, 0, 0, &error);
+
+		if (error)
+		{
+			g_warning("KzBookmark: Regular Expression Error");
+			g_error_free(error);
+			return NULL;
+		}
+
+		if (egg_regex_match(egg_regex, text, -1, 0) > 0)
+		{
+			match = prop;
+			break;
+		}
+		egg_regex_free(egg_regex);
+	}
+
+	if(match)
+	{
+		gboolean urlencode = match->urlencode;
+		gchar *uri = match->uri;
+		gchar *encode = match->encode;
+		gchar **strings = NULL;
+		gchar *url_strings[MAX_GROUPSTRING];
+		guint i, n = 0;
+		GString *str;
+
+		strings = egg_regex_fetch_all(egg_regex, text);
+	
+		if (strings)
+		{
+			while (strings[n] && n < MAX_GROUPSTRING)
+			{
+				GError *e = NULL;
+				gchar *encode_string = NULL;
+
+				if (encode)
+				{
+					encode_string = 
+						g_convert(strings[n],
+							  strlen(strings[n]),
+							  encode, "UTF-8",
+							  NULL, NULL,
+							  &e);
+				}
+				
+				if (e || !encode)
+				{
+					encode_string =
+					    g_strdup(strings[n]);
+					if (e)
+						g_error_free(e);
+				}
+				if (urlencode)	
+				{
+					url_strings[n] = 
+						url_encode(encode_string);
+				}
+				else
+				{
+					url_strings[n] = g_strdup(encode_string);
+				}
+
+				g_free(encode_string);
+				n++;
+			}
+		}
+		g_strfreev(strings);	
+
+		str = g_string_new_len(uri, strlen(uri));
+		if (strstr(uri, "%s"))
+		{
+			gchar *encode_string = NULL;
+			gchar *url;
+			GError *e = NULL;
+
+			if (encode)
+			{
+				encode_string = 
+					g_convert(text,
+						  strlen(text),
+						  encode, "UTF-8",
+						  NULL, NULL,
+						  &e);
+			}
+				
+			if (e || !encode)
+			{
+				encode_string =
+				    g_strdup(text);
+				if (e)
+					g_error_free(e);
+			}
+			if (urlencode)	
+			{
+				url = url_encode(encode_string);
+			}
+			else
+			{
+				url = g_strdup(encode_string);
+			}
+			g_string_printf(str, uri, url);
+			g_free(url);
+			g_free(encode_string);
+		}
+
+		for (i = 0; i < n; i++)
+		{
+			gchar *pos;
+			gchar *index;
+			index = g_strdup_printf("\\%d", i);
+			pos = strstr(str->str, index); 
+			if (pos)
+			{
+				gssize p = pos - str->str;
+				str = g_string_erase(str, p, strlen(index));
+				str = g_string_insert(str, p, url_strings[i]); 
+			}
+			g_free(index);
+		}
+		
+		smart_uri = g_strndup(str->str, str->len);	
+		g_string_free(str, TRUE);
+
+		egg_regex_free(egg_regex);
+	}
+	else
+	{
+                /* match nothing so nothing to do... */
+		smart_uri = g_strdup(text);
+	}
+
+	return smart_uri;
+}
+
+
 static void
 kz_bookmark_real_insert_child (KzBookmark *bookmark,
 			       KzBookmark *child,
@@ -1231,6 +1526,7 @@ kz_bookmark_load_start (KzBookmark *bookmark)
 {
 	KzIO *io;
 	const gchar *uri;
+	guint id;
 
 	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
 
@@ -1249,9 +1545,14 @@ kz_bookmark_load_start (KzBookmark *bookmark)
 	kz_bookmark_remove_all(KZ_BOOKMARK(bookmark));
 
 	io = kz_io_new(uri);
-	g_signal_connect(G_OBJECT(io), "io_completed",
-			 G_CALLBACK(cb_io_load_complete),
-			 bookmark);
+	id = g_signal_connect(G_OBJECT(io), "io_completed",
+			      G_CALLBACK(cb_io_load_complete),
+			      bookmark);
+	
+	g_object_set_qdata(G_OBJECT(bookmark), io_quark, io);
+	g_object_set_qdata(G_OBJECT(bookmark), io_signal_id_quark,
+			   GUINT_TO_POINTER(id));
+
 	kz_io_load_to_buffer(io);
 
 	return TRUE;
@@ -1287,7 +1588,6 @@ kz_bookmark_load (KzBookmark *bookmark)
 }
 
 
-#warning FIXME! we should use KzIO for writing contents.
 void
 kz_bookmark_save (KzBookmark *bookmark)
 {
@@ -1360,6 +1660,9 @@ ERROR:
 		 G_CALLBACK(cb_io_load_complete),
 		 bookmark);
 	kz_bookmark_set_state(bookmark, KZ_BOOKMARK_STATE_NORMAL);
+	g_object_set_qdata(G_OBJECT(bookmark), io_quark, NULL);
+	g_object_set_qdata(G_OBJECT(bookmark), io_signal_id_quark,
+			   GPOINTER_TO_UINT(0));
 
 	if (error)
 	{
@@ -1442,6 +1745,37 @@ kz_bookmark_get_state (KzBookmark *bookmark)
 }
 
 
+void
+kz_bookmark_set_smart_list (KzBookmark *bookmark, GList *list)
+{
+	GList *old_list;
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+	
+	if (!kz_bookmark_is_smart(bookmark)) return;
+	
+	old_list = g_object_get_qdata(G_OBJECT(bookmark),
+				      smart_list_quark);
+	free_smart_list(old_list);
+
+	g_object_set(G_OBJECT(bookmark), "smart-list", list, NULL);
+}
+
+
+const GList *
+kz_bookmark_get_smart_list (KzBookmark *bookmark)
+{
+	const GList *list;
+
+	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);
+	
+	if (!kz_bookmark_is_smart(bookmark)) return NULL;
+
+	list = g_object_get_qdata(G_OBJECT(bookmark),
+				  smart_list_quark);
+
+	return list;
+}
+
 static gboolean
 kz_bookmark_is_loading_all_children (KzBookmark *bookmark)
 {
