@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark.h,v 1.17 2004/04/12 15:07:45 ikezoe Exp $
+ *  $Id: kz-bookmark.h,v 1.19 2004/07/15 13:19:53 ikezoe Exp $
  */
 
 #ifndef __KZ_BOOKMARK_H__
@@ -64,6 +64,15 @@ typedef enum {
 	KZ_BOOKMARK_STATE_SAVING
 } KzBookmarkState;
 
+typedef struct _KzBookmarkSmartProperty
+{
+	gchar    *regex;
+	gchar    *uri;
+	gchar    *encode;
+	gboolean  urlencode;
+} KzBookmarkSmartProperty;
+
+
 struct _KzBookmark
 {
 	GObject         parent;
@@ -206,6 +215,20 @@ void		kz_bookmark_save		 (KzBookmark *bookmark);
 
 KzBookmarkState kz_bookmark_get_state		 (KzBookmark *bookmark);
 
+/* smartbookmark interface */
+gboolean	kz_bookmark_is_smart		 (KzBookmark *bookmark);
+void		kz_bookmark_append_smart_property(KzBookmark *bookmark,
+						  const gchar *regex,
+						  const gchar *uri,
+						  const gchar *encode,
+						  gboolean urlencode);
+void 		kz_bookmark_remove_smart_property(KzBookmark *bookmark);
+
+gchar	       *kz_bookmark_get_smart_uri        (KzBookmark *bookmark,
+						  const gchar *text);
+const GList    *kz_bookmark_get_smart_list	 (KzBookmark *bookmark);
+void            kz_bookmark_set_smart_list	 (KzBookmark *bookmark,
+						  GList *list);
 
 G_END_DECLS
 
