@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmarks-view.c,v 1.9 2004/05/05 02:25:57 ikezoe Exp $
+ *  $Id: kz-bookmarks-view.c,v 1.13 2004/07/23 03:20:55 ikezoe Exp $
  */
 
 #include "kz-bookmarks-view.h"
@@ -208,6 +208,7 @@ kz_bookmarks_view_new (void)
 				   G_TYPE_STRING,
 				   G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(view), GTK_TREE_MODEL(store));
+	g_object_unref(store);
 
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), TRUE);
 
@@ -535,6 +536,17 @@ insert_bookmark(KzBookmarksView *view,
 						 NULL);
 		gtk_widget_destroy(image);
 	}
+	else if (kz_bookmark_is_smart(bookmark))
+	{
+		GtkWidget *image;
+
+		image = gtk_image_new_from_stock(KZ_STOCK_SMART_BOOKMARK,
+						 GTK_ICON_SIZE_MENU);
+		favicon = gtk_widget_render_icon(image, KZ_STOCK_SMART_BOOKMARK,
+						 GTK_ICON_SIZE_MENU,
+						 NULL);
+		gtk_widget_destroy(image);
+	}
 	else if (kz_bookmark_is_separator(bookmark))
 	{
 		favicon = NULL;
@@ -636,10 +648,10 @@ remove_bookmark(KzBookmarksView *view,
 	GtkTreeModel *model;
 	GtkTreePath *path;
 	GtkTreeIter iter;
-
+	
 	treeview = GTK_TREE_VIEW(view);
 	model = gtk_tree_view_get_model(treeview);
-
+	gtk_widget_grab_focus(GTK_WIDGET(treeview));
 	/* get iter */
 	if (bookmark)
 	{
@@ -648,8 +660,8 @@ remove_bookmark(KzBookmarksView *view,
 
 		gtk_tree_model_get_iter(model, &iter, path);
 		gtk_tree_path_free(path);
-	}
-
+	}	
+	
 	/* remove row */
 	gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
 }
@@ -1119,7 +1131,7 @@ cb_drag_data_received (GtkWidget *widget,
 		if (!parent) goto ERROR;
 
 		kz_bookmark_insert_before(parent, src, dest);
-
+		gtk_drag_finish(context, TRUE, FALSE, time);
 		break;
 	default:
 		break;
