@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-xbel.c,v 1.39 2004/05/15 13:27:30 ikezoe Exp $
+ *  $Id: kz-xbel.c,v 1.42 2004/07/15 13:19:53 ikezoe Exp $
  */
 
 #include "kazehakase.h"
@@ -172,6 +172,7 @@ kz_xbel_from_string (KzBookmark *bookmark,
 		node = kz_xml_element_node_new("xbel");
 		kz_xml_node_set_attr(node, "version", "1.0");
 		kz_xml_node_set_attr(node, "folded", "none");	
+		kz_xml_node_set_attr(node, "xmlns", KAZEHAKASE_URI"2004");	
 		/* kz_xml_node_set_attr(node, "id", "????????"); */
 		kz_xml_node_append_child(xml->root, node);
 
@@ -272,72 +273,6 @@ kz_xbel_notify (GObject *object, GParamSpec *pspec, KzXML *xml)
 }
 
 
-static void
-parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
-{
-	KzXMLNode *node;
-
-	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-
-	if (!kz_xml_node_is_element(parent)) return;
-
-	for (node = kz_xml_node_first_child(parent);
-	     node;
-	     node = kz_xml_node_next(node))
-	{
-		if (!kz_xml_node_is_element(node)) continue;
-
-		if (kz_xml_node_name_is(node, "metadata"))
-		{
-			const gchar *owner;
-
-			/* We should ignore other owners such as Galeon. */
-			owner = kz_xml_node_get_attr(node, "owner");
-			if (!owner || strcmp(owner, KAZEHAKASE_URI))
-				continue;
-
-			parse_metadata_node(bookmark, node);
-			/*
-			if (kz_bookmark_get_owner(bookmark))
-			{
-				g_warning("metadata owner attribute is duplicated!");
-				continue;
-			}
-			owner = kz_xml_node_get_attr(node, "owner");
-			kz_bookmark_set_owner(folder, owner);
-			*/
-		}
-		else if (kz_xml_node_name_is(node, "location"))
-		{
-			/*
-			 * "location" element is already parsed at
-			 * parse_child_node().
-			 */
-			if (!kz_bookmark_is_file(bookmark))
-			{
-				g_warning("kzXBEL: The bookmark has location, "
-					  "but it's not a folder element!");
-			}
-		}
-		else if (kz_xml_node_name_is(node, "update_interval"))
-		{
-			gchar *interval_str;
-			guint interval;
-
-			if (kz_bookmark_get_interval(bookmark))
-			{
-				g_warning("update_interval element is duplicated!");
-				continue;
-			}
-			interval_str = kz_xml_node_to_str(node);
-			interval = atoi(interval_str);
-			kz_bookmark_set_interval(bookmark, interval);
-			g_free(interval_str);
-		}
-	}	
-}
-
-
 static KzXMLNode *
 xml_node_get_named_node (KzXMLNode *parent, const gchar *name)
 {
@@ -358,6 +293,127 @@ xml_node_get_named_node (KzXMLNode *parent, const gchar *name)
 }
 
 
+static void
+xml_node_set_location (KzXMLNode *node, KzBookmark *bookmark);
+static void
+xml_node_set_interval (KzXMLNode *node, KzBookmark *bookmark);
+
+static void
+parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
+{
+	KzXMLNode *node;
+
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+
+	if (!kz_xml_node_is_element(parent)) return;
+
+	for (node = kz_xml_node_first_child(parent);
+	     node;
+	     node = kz_xml_node_next(node))
+	{
+		if (!kz_xml_node_is_element(node)) continue;
+
+		if (kz_xml_node_name_is(node, "metadata"))
+		{
+			const GList *attrs, *a_node;
+			const gchar *owner;
+			const gchar *smart_regex  = NULL;
+			const gchar *smart_uri    = NULL;
+			const gchar *smart_encode = NULL;
+			gboolean urlencode = FALSE;
+			gint interval;
+			
+			/* We should ignore other owners such as Galeon. */
+			owner = kz_xml_node_get_attr(node, "owner");
+			if (!owner || strcmp(owner, KAZEHAKASE_URI))
+				continue;
+			
+			attrs = kz_xml_node_get_attrs(node);
+
+			for (a_node = attrs; a_node; a_node = g_list_next(a_node))
+			{
+				KzXMLAttr *attr = a_node->data;
+
+				if (!strncmp(attr->name, "kz:update_interval", 118))
+				{
+					interval = atoi(attr->value);
+					kz_bookmark_set_interval(bookmark, interval);
+				}
+				else if (!strncmp(attr->name, "kz:smart_regex", 14))
+				{
+					smart_regex = attr->value;
+				}
+				else if (!strncmp(attr->name, "kz:smart_uri", 12))
+				{
+					smart_uri = attr->value;
+				}
+				else if (!strncmp(attr->name, "kz:smart_encode", 15))
+				{
+					smart_encode = attr->value;
+				}
+				else if (!strncmp(attr->name, "kz:smart_urlencode", 16))
+				{
+					urlencode = TRUE;
+				}
+			}
+
+			if (smart_regex && smart_uri)
+			{
+				 kz_bookmark_append_smart_property(
+					bookmark,
+					smart_regex, smart_uri, 
+					smart_encode, urlencode);
+			}
+
+			{
+				/* 
+				 * for compatibility, 
+				 * we need to load "update_interval" 
+				 * from node.
+				 */
+				KzXMLNode *interval_node;
+				interval_node = xml_node_get_named_node(node, 
+							   "update_interval");
+				if (interval_node)
+				{
+					gchar *str;
+					str = kz_xml_node_to_str(interval_node);		
+					interval = atoi(str);
+					kz_bookmark_set_interval(bookmark,
+								 interval);
+					kz_xml_node_set_attr(node,
+							     "kz:update_interval",
+							     str);
+					g_free(str);
+#if 0
+					/* clean "update_interval" node */
+					kz_xml_node_remove_child(node, interval_node);
+					kz_xml_node_unref(interval_node);
+#endif
+				}
+			}
+			/*
+			if (kz_bookmark_get_owner(bookmark))
+			{
+				g_warning("metadata owner attribute is duplicated!");
+				continue;
+			}
+			owner = kz_xml_node_get_attr(node, "owner");
+			kz_bookmark_set_owner(folder, owner);
+			*/
+		}
+		else if (kz_xml_node_name_is(node, "location"))
+		{
+			g_warning("kzXBEL: \"location\" element is obsolete");
+		}
+		else if (kz_xml_node_name_is(node, "update_interval"))
+		{
+			g_warning("KzXBEL: \"update_interval\" element is obsolete!");
+		}
+	}	
+}
+
+
 static KzXMLNode *
 xml_node_find_metadata_node (KzXMLNode *parent)
 {
@@ -397,6 +453,9 @@ xml_node_has_location (KzXMLNode *parent)
 	child = xml_node_find_metadata_node(parent);
 	if (!child) return FALSE;
 
+	if (kz_xml_node_get_attr(child, "kz:location"))
+		return TRUE;
+
 	gchild = xml_node_get_named_node(child, "location");
 	if (gchild) return TRUE;
 
@@ -408,6 +467,8 @@ static gchar *
 xml_node_get_location (KzXMLNode *parent)
 {
 	KzXMLNode *child, *gchild;
+	const gchar *attr;
+	gchar *location;
 
 	g_return_val_if_fail(parent, FALSE);
 
@@ -417,10 +478,24 @@ xml_node_get_location (KzXMLNode *parent)
 	child = xml_node_find_metadata_node(parent);
 	if (!child) return NULL;
 
+	/* Does metadata node have location attr? */
+	attr = kz_xml_node_get_attr(child, "kz:location");
+	if (attr)
+		return g_strdup(attr);
+
 	gchild = xml_node_get_named_node(child, "location");
 	if (!gchild) return NULL;
 
-	return kz_xml_node_to_str(gchild);
+	location = kz_xml_node_to_str(gchild);
+	/* set location into metadata attribute */
+	kz_xml_node_set_attr(child, 
+			     "kz:location", location);
+#if 0
+	/* remove "location" element */
+	kz_xml_node_remove_child(child, gchild);
+	kz_xml_node_unref(gchild);
+#endif
+	return location;
 }
 
 
@@ -480,6 +555,7 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 				folder = kz_bookmark_file_new(location,
 							      NULL, NULL);
 				kz_bookmark_load_start(folder);
+				g_free(location);
 			}
 			else
 			{
@@ -497,14 +573,6 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			parse_child_node(folder, node);
 			BOOKMARK_UNSET_BUILDING(folder);
 
-			/* Does this need? */
-			/*
-			if (kz_bookmark_is_file(folder))
-			{
-				kz_bookmark_remove_all(folder);
-				kz_bookmark_load_start(folder);
-			}
-			 */ 
 			g_object_unref(folder);
 		}
 		else if (kz_xml_node_name_is(node, "bookmark"))
@@ -516,8 +584,13 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			g_object_set_qdata(G_OBJECT(child_bookmark),
 					   node_quark, node);
 			kz_bookmark_set_link(child_bookmark, uri);
-			kz_bookmark_append(bookmark, child_bookmark);
 			parse_child_node(child_bookmark, node);
+			/*
+			 * To append child is after parsing child's nodes,
+			 * because smart bookmark action needs bookmark title 
+			 * in child's nodes.
+			 */
+			kz_bookmark_append(bookmark, child_bookmark);
 			BOOKMARK_UNSET_BUILDING(bookmark);
 			g_object_unref(child_bookmark);
 		}
@@ -642,40 +715,17 @@ static void
 xml_node_set_location (KzXMLNode *node, KzBookmark *bookmark)
 {
 	const gchar *location;
-	KzXMLNode *metadata_node, *location_node,
-		  *text_node, *space;
+	KzXMLNode *metadata_node;
 
 	location = kz_bookmark_get_location(bookmark);
 	g_return_if_fail(location);
 
-	metadata_node = xml_node_append_metadata_node(node);
-	location_node = xml_node_get_named_node(metadata_node, "location");
+	metadata_node = xml_node_find_metadata_node(node);
+	
+	if (!metadata_node)
+		metadata_node = xml_node_append_metadata_node(node);
 
-	if (location_node)
-	{
-		KzXMLNode *child;
-
-		/* remove all child node */
-		child = kz_xml_node_first_child(location_node);
-		while (child)
-		{
-			KzXMLNode *next = kz_xml_node_next(child);
-			kz_xml_node_remove_child(location_node, child);
-			kz_xml_node_unref(child);
-			child = next;
-		}
-	}
-	else
-	{
-		location_node = kz_xml_element_node_new("location");
-		kz_xml_node_append_child(metadata_node, location_node);
-	}
-
-	text_node = kz_xml_text_node_new(location);
-	kz_xml_node_append_child(location_node, text_node);
-
-	space = kz_xml_text_node_new("\n");
-	kz_xml_node_append_child(metadata_node, space);
+	kz_xml_node_set_attr(metadata_node, "kz:location", location);
 }
 
 
@@ -683,46 +733,125 @@ static void
 xml_node_set_interval (KzXMLNode *node, KzBookmark *bookmark)
 {
 	gchar *interval;
-	KzXMLNode *metadata_node, *interval_node,
-		  *text_node, *space;
+	KzXMLNode *metadata_node;
 
 	interval = g_strdup_printf("%d", kz_bookmark_get_interval(bookmark));
 
-	metadata_node = xml_node_append_metadata_node(node);
-	interval_node = xml_node_get_named_node(metadata_node,
-						"update_interval");
+	metadata_node = xml_node_find_metadata_node(node);
+	
+	if (!metadata_node)
+		metadata_node = xml_node_append_metadata_node(node);
 
 	/* update_interval */
-	if (interval_node)
-	{
-		KzXMLNode *child;
-
-		/* remove all child node */
-		child = kz_xml_node_first_child(interval_node);
-		while (child)
-		{
-			KzXMLNode *next = kz_xml_node_next(child);
-			kz_xml_node_remove_child(interval_node, child);
-			kz_xml_node_unref(child);
-			child = next;
-		}
-	}
-	else
-	{
-		interval_node = kz_xml_element_node_new("update_interval");
-		kz_xml_node_append_child(metadata_node, interval_node);
-	}
-
-	text_node = kz_xml_text_node_new(interval);
-	kz_xml_node_append_child(interval_node, text_node);
-
-	space = kz_xml_text_node_new("\n");
-	kz_xml_node_append_child(metadata_node, space);
+	kz_xml_node_set_attr(metadata_node, "kz:update_interval", interval);
 
 	g_free(interval);
 }
 
 
+static void
+xml_node_set_smart_list (KzXMLNode *node,
+			 KzBookmark *bookmark,
+		       	 const GList *smart_list)
+{
+	const GList *smart_node;	
+	KzXMLNode *info_node, *child_node;
+
+	/* get the info node, create it if not exist */
+	info_node = xml_node_get_named_node(node, "info");
+	if (!info_node)
+	{
+		KzXMLNode *space;
+
+		info_node = kz_xml_element_node_new("info");
+		kz_xml_node_append_child(node, info_node);
+
+		/* space1 */
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(info_node, space);
+
+		/* space4 */
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(node, space);
+	}
+
+	/* remove all previous smart properties */
+	child_node = kz_xml_node_first_child(info_node);
+	while (child_node)
+	{
+		if (kz_xml_node_name_is(child_node, "metadata"))
+		{
+			const gchar *owner, *smart_regex;
+			owner = kz_xml_node_get_attr(child_node, "owner");
+			smart_regex = kz_xml_node_get_attr(child_node,
+							   "kz:smart_regex");
+			if (owner && 
+			   !strcmp(owner, KAZEHAKASE_URI) &&
+			    smart_regex)
+			{
+				KzXMLNode *tmp = child_node;
+				child_node = kz_xml_node_next(child_node);
+				kz_xml_node_remove_child(info_node,
+							 tmp);
+				kz_xml_node_unref(tmp);
+
+				/* space is also removed */
+				if (kz_xml_node_is_space(child_node))
+				{
+					KzXMLNode *tmp = child_node;
+					child_node = kz_xml_node_next(child_node);
+					kz_xml_node_remove_child(info_node,
+								 tmp);
+					kz_xml_node_unref(tmp);
+				}
+			}
+			else
+				child_node = kz_xml_node_next(child_node);
+		}
+		else
+			child_node = kz_xml_node_next(child_node);
+	}
+
+	/* append new smart properties */
+	for (smart_node = smart_list; smart_node; smart_node = g_list_next(smart_node))
+	{
+		KzBookmarkSmartProperty *prop;
+		KzXMLNode *metadata_node, *space;
+		
+		metadata_node = kz_xml_element_node_new("metadata");
+		kz_xml_node_set_attr(metadata_node, "owner", KAZEHAKASE_URI);
+		kz_xml_node_append_child(info_node, metadata_node);
+
+		/* space */
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(info_node, space);
+
+		prop = smart_node->data;
+
+		if (prop->regex)
+		{
+			kz_xml_node_set_attr(metadata_node,
+					     "kz:smart_regex", prop->regex);
+		}
+		if (prop->uri)
+		{
+			kz_xml_node_set_attr(metadata_node,
+					     "kz:smart_uri", prop->uri);
+		}
+		if (prop->encode)
+		{
+			kz_xml_node_set_attr(metadata_node,
+					     "kz:smart_encode", prop->encode);
+		}
+		if (prop->urlencode)
+		{
+			kz_xml_node_set_attr(metadata_node,
+					     "kz:smart_urlencode", "yes");
+		}
+	}
+}
+
+
 static KzXMLNode *
 create_xml_node (KzBookmark *bookmark)
 {
@@ -778,6 +907,27 @@ create_xml_node (KzBookmark *bookmark)
 
 		xml_node_append_title (node, bookmark);
 	}
+	else if (kz_bookmark_is_smart(bookmark))
+	{
+		const GList *smart_list;
+		const gchar *uri = kz_bookmark_get_link(bookmark);
+		
+		smart_list = kz_bookmark_get_smart_list(bookmark);
+	
+		node = kz_xml_element_node_new("bookmark");
+		if (uri)
+			kz_xml_node_set_attr(node, "href", uri);
+		if (smart_list)
+		{
+			xml_node_set_smart_list(node, 
+						bookmark,
+						smart_list);
+		}
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(node, space);
+
+		xml_node_append_title (node, bookmark);
+	}
 	else
 	{
 		const gchar *uri = kz_bookmark_get_link(bookmark);
@@ -1084,6 +1234,12 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec)
 		xml_node_set_interval(node, bookmark);
 		/* xml_node_set_metadata(node, "update_interval", str); */
 	}
+	else if (!strcmp(prop, "smart-list"))
+	{
+		const GList *smart_list;
+		smart_list =  g_value_get_pointer(&value);
+		xml_node_set_smart_list(node, bookmark, smart_list);
+	}
 }
 
 
