@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-menu.c,v 1.30 2004/05/08 14:45:03 ikezoe Exp $
+ *  $Id: kz-bookmark-menu.c,v 1.38 2004/07/05 09:24:06 ikezoe Exp $
  */
 
 #include "kz-bookmark-menu.h"
@@ -35,17 +35,17 @@
 #define KZ_BOOKMARK_MENU_TOOLTIP_KEY  "KzBookmarkMenu::ToolTips"
 #define KZ_BOOKMARK_MENU_EMPTY_KEY    "KzBookmarkMEnu::Empty"
 #define KZ_BOOKMARK_MENU_PROXY_KEY    "KzBookmarkMenu::IsFolderProxy"
+#define KZ_BOOKMARK_MENU_BUTTON_KEY   "KzBookmarkMenu::MouseButton"
 
 
-static void       bookmark_folder_open_all       (KzBookmark     *bookmark,
-						  KzWindow       *kz);
-
 static gboolean   cb_menu_item_button_press      (GtkWidget      *widget,
 						  GdkEventButton *event,
 						  KzWindow       *kz);
 static gboolean   cb_menu_item_button_release    (GtkWidget      *widget,
 						  GdkEventButton *event,
 						  KzWindow       *kz);
+static void       cb_menu_item_activate          (GtkWidget      *widget,
+						  KzWindow       *kz);
 
 static void       connect_bookmark_signals       (GtkWidget      *widget,
 						  KzBookmark     *item);
@@ -61,7 +61,6 @@ static void       disconnect_folder_signals      (KzBookmark     *folder,
 static void       disconnect_menu_signals        (KzBookmark     *folder,
 						  GtkMenuShell   *shell);
 
-
 GtkWidget *
 kz_bookmark_menu_create_menuitem (KzBookmark *bookmark, KzWindow *kz,
 				  gboolean is_folder_proxy)
@@ -116,12 +115,15 @@ kz_bookmark_menu_create_menuitem (KzBookmark *bookmark, KzWindow *kz,
 	g_signal_connect(G_OBJECT(menu_item), "button_release_event",
 			 G_CALLBACK(cb_menu_item_button_release),
 			 kz);
+	g_signal_connect(G_OBJECT(menu_item), "activate",
+			 G_CALLBACK(cb_menu_item_activate),
+			 kz);
 
 	/* submenu */
 	if (kz_bookmark_is_folder(bookmark) && !is_folder_proxy)
 	{
 		GtkWidget *submenu, *image;
-
+	
 		submenu = kz_bookmark_menu_create_submenu(bookmark, kz);
 		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);
 		image = gtk_image_new_from_stock(KZ_STOCK_FOLDER,
@@ -166,7 +168,6 @@ GtkWidget *
 kz_bookmark_menu_create_submenu (KzBookmark *folder, KzWindow *kz)
 {
 	GtkWidget *submenu, *menu_item;
-	const gchar *link, *title;
 
 	g_return_val_if_fail(KZ_IS_BOOKMARK(folder), NULL);
 	g_return_val_if_fail(kz_bookmark_is_folder(folder), NULL);
@@ -175,11 +176,6 @@ kz_bookmark_menu_create_submenu (KzBookmark *folder, KzWindow *kz)
 	/* create submenu  */
 	submenu = gtk_menu_new();
 
-	/* channel link */
-	link  = kz_bookmark_get_link(folder);
-	title = kz_bookmark_get_document_title(folder);
-	if (!title || !* title) title = kz_bookmark_get_title(folder);
-
 	menu_item = kz_bookmark_menu_create_menuitem(folder, kz, TRUE);
 	if (menu_item)
 	{
@@ -200,7 +196,7 @@ kz_bookmark_menu_create_submenu (KzBookmark *folder, KzWindow *kz)
 
 
 void
-kz_bookmark_menu_append_menuitems (GtkMenuShell *shell,KzWindow *kz, 
+kz_bookmark_menu_append_menuitems (GtkMenuShell *shell, KzWindow *kz, 
 				   KzBookmark *folder)
 {
 	KzBookmark *prev_folder;
@@ -308,84 +304,11 @@ kz_bookmark_menu_remove_menuitems (GtkMenuShell *shell, KzWindow *kz)
 }
 
 
-static void
-bookmark_folder_open_all (KzBookmark *bookmark, KzWindow *kz)
-{
-	GList *node, *items;
-	KzBookmark *item = NULL;
-	GtkWidget *parent = NULL;
-	const gchar *channel_uri;
-	gboolean confirm;
-
-	/* confirmation */
-	KZ_CONF_GET("Global", "confirm_open_bookmarks", confirm, BOOL);
-	if (confirm)
-	{
-		GtkWidget *dialog;
-		gint res;
-		dialog = gtk_message_dialog_new(GTK_WINDOW(kz),
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_QUESTION,
-						GTK_BUTTONS_YES_NO,
-						_("Open all bookmarks in this bookmark folder ?"));
-		gtk_dialog_set_default_response(GTK_DIALOG(dialog),
-						GTK_RESPONSE_YES);
-		res = gtk_dialog_run(GTK_DIALOG(dialog));
-		gtk_widget_destroy(dialog);
-	
-		if (res == GTK_RESPONSE_NO) return;
-	}
-	
-	channel_uri = kz_bookmark_get_link(bookmark);
-	if (channel_uri)
-		parent = kz_window_open_new_tab(kz, channel_uri);
-
-	items = kz_bookmark_get_children(bookmark);
-	for (node = items; node; node = g_list_next(node))
-	{
-		const gchar *uri;
-
-		item = node->data;
-		uri = kz_bookmark_get_link(item);
-		if (uri)
-			kz_window_open_new_tab_with_parent(kz, uri, parent);
-	}
-	g_list_free(items);
-}
-
-
 /*****************************************************************************
  *                                                                           *
  *                                Callbacks                                  *
  *                                                                           *
  *****************************************************************************/
-#if 0
-static void 
-cb_bookmark_load_start (KzBookmark *meta, KzBookmarkMenu *bkmkmenu)
-{
-	g_return_if_fail(KZ_IS_BOOKMARK(meta));
-	g_return_if_fail(KZ_IS_BOOKMARK_MENU(bkmkmenu));
-
-	gtk_image_set_from_stock(GTK_IMAGE(bkmkmenu->image),
-				 KZ_STOCK_RED,
-				 KZ_ICON_SIZE_BOOKMARK_MENU);
-}
-
-
-static void 
-cb_bookmark_load_stop (KzBookmark *bookmark, KzBookmarkMenu *menu)
-{
-	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-	g_return_if_fail(KZ_IS_BOOKMARK_MENU(menu));
-
-	kz_bookmark_menu_create_submenu(menu, bookmark, NULL);
-
-	gtk_image_set_from_stock(GTK_IMAGE(menu->image),
-				 KZ_STOCK_GREEN,
-				 KZ_ICON_SIZE_BOOKMARK_MENU);
-}
-#endif
-
 
 static gint
 find_menu_item (gconstpointer a, gconstpointer b)
@@ -490,24 +413,48 @@ disconnect_menuitem_signals (GtkWidget *widget, KzBookmark *item)
 }
 
 
-static gboolean
-cb_menu_item_button_press (GtkWidget *widget, GdkEventButton *event,
-			   KzWindow *kz)
+static void 
+cb_menu_item_activate (GtkWidget *widget, KzWindow *kz)
 {
 	KzBookmark *item;
 	const gchar *uri;
+	gint button;
+	gboolean is_folder_proxy;
 
-	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
+	g_return_if_fail(KZ_IS_WINDOW(kz));
 
 	item = g_object_get_data(G_OBJECT(widget),
 				 KZ_BOOKMARK_MENU_BOOKMARK_KEY);
-	g_return_val_if_fail(item, FALSE);
+	g_return_if_fail(item);
+
+	button = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget),
+				   KZ_BOOKMARK_MENU_BUTTON_KEY));
+
+	is_folder_proxy = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget),
+			      KZ_BOOKMARK_MENU_PROXY_KEY));
+
+	if (kz_bookmark_is_folder(item) && !is_folder_proxy)
+		return;
 
 	uri = kz_bookmark_get_link(item);
-
-	switch (event->button)
+	
+	switch (button)
 	{
-	case 1:
+	case 2:
+		if (kz_bookmark_is_folder(item))
+		{
+			GtkAction *action;
+			kz_actions_set_bookmark_for_action(kz, item);
+			action = gtk_action_group_get_action(kz->actions,
+							     "OpenAllBookmarks");
+			if (action)
+				gtk_action_activate(action);
+			kz_actions_set_bookmark_for_action(kz, NULL);
+		}
+		else if (uri)
+			kz_window_open_new_tab(kz, uri);
+		break;
+	default:
 		if (uri)
 		{
 			kz_window_load_url(kz, uri);
@@ -521,13 +468,71 @@ cb_menu_item_button_press (GtkWidget *widget, GdkEventButton *event,
 				(KZ_BOOKMARK_EDITOR(editor), item);
 			gtk_widget_show(editor);
 		}
-		return TRUE;
 		break;
+	}
+
+	button = 0;
+	g_object_set_data(G_OBJECT(widget),
+			  KZ_BOOKMARK_MENU_BUTTON_KEY,
+			  GINT_TO_POINTER(button));
+}
+
+
+
+static gboolean
+cb_menu_item_button_press (GtkWidget *widget, GdkEventButton *event,
+			   KzWindow *kz)
+{
+	return FALSE;
+}
+
+
+static gboolean
+cb_menu_item_button_release (GtkWidget *widget, GdkEventButton *event,
+			    KzWindow *kz)
+{
+	KzBookmark *item;
+	const gchar *uri;
+	gboolean keep = FALSE;
+	gint button = event->button;
+
+	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
+
+	item = g_object_get_data(G_OBJECT(widget),
+				 KZ_BOOKMARK_MENU_BOOKMARK_KEY);
+	g_return_val_if_fail(item, FALSE);
+
+	uri = kz_bookmark_get_link(item);
+
+	g_object_set_data(G_OBJECT(widget),
+			  KZ_BOOKMARK_MENU_BUTTON_KEY,
+			  GINT_TO_POINTER(button));
+
+	switch (button)
+	{
 	case 2:
+		KZ_CONF_GET("Global", "keep_bookmark_menu", keep, BOOL);
+		if (!keep)
+			return FALSE;
+
 		if (kz_bookmark_is_folder(item))
-			bookmark_folder_open_all(item, kz);
+		{
+			GtkAction *action;
+			kz_actions_set_bookmark_for_action(kz, item);
+			action = gtk_action_group_get_action(kz->actions,
+							     "OpenAllBookmarks");
+			if (action)
+				gtk_action_activate(action);
+			kz_actions_set_bookmark_for_action(kz, NULL);
+		}
 		else if (uri)
 			kz_window_open_new_tab(kz, uri);
+		/* reset release button state */
+		button = 0;
+		g_object_set_data(G_OBJECT(widget),
+				  KZ_BOOKMARK_MENU_BUTTON_KEY,
+				  GINT_TO_POINTER(button));
+
 		return TRUE;
 		break;
 	case 3:
@@ -544,21 +549,6 @@ cb_menu_item_button_press (GtkWidget *widget, GdkEventButton *event,
 }
 
 
-static gboolean
-cb_menu_item_button_release (GtkWidget *widget, GdkEventButton *event,
-			     KzWindow *kz)
-{
-	gboolean keep = FALSE;
-
-	if (event->button == 1)
-		return FALSE;
-
-	KZ_CONF_GET("Global", "keep_bookmark_menu", keep, BOOL);
-	if (keep)
-		return TRUE;
-	return FALSE;
-}
-
 /*****************************************************************************
  *                                                                           *
  *                               for  menu                                   *
