@@ -32,8 +32,6 @@
 #include "kz-favicon.h"
 #include "kz-input-event-box.h"
 
-#define PROGRESSBAR_HEIGHT 16
-
 
 enum {
 	PROP_0,
@@ -51,6 +49,15 @@ struct _KzTabLabelPriv
 	gulong favicon_signal;
 };
 
+#warning FIXME!
+
+const gchar *label_color[] =
+{
+	"#000000",
+	"#ff0000",
+	"#22aa44",
+};
+
 /* object class */
 static void     kz_tab_label_class_init         (KzTabLabelClass *klass);
 static void     kz_tab_label_init               (KzTabLabel *kztab);
@@ -70,7 +77,9 @@ static void     kz_tab_label_get_property       (GObject *object,
 
 /* widget class */
 static void     kz_tab_label_realize            (GtkWidget *widget);
+#if 0
 static void     kz_tab_label_unrealize          (GtkWidget *widget);
+#endif
 static gboolean kz_tab_label_button_press       (GtkWidget *widget,
 						 GdkEventButton *event);
 static gboolean kz_tab_label_motion_notify      (GtkWidget *widget,
@@ -118,7 +127,7 @@ static void     cb_close_button_state_changed   (GtkWidget *widget,
 static void cb_title_changed (GtkMozEmbed *embed, KzTabLabel *kztab);
 static void cb_net_start     (GtkMozEmbed *embed, KzTabLabel *kztab);
 static void cb_net_stop      (GtkMozEmbed *embed, KzTabLabel *kztab);
-static void cb_destroy       (GtkObject *object, KzTabLabel *kztab);
+static void cb_destroy       (GtkObject *object,  KzTabLabel *kztab);
 static void cb_net_state_all (GtkMozEmbed *embed, const char *aURI,
 			      gint state, gint status,
 			      KzTabLabel *kztab);
@@ -172,7 +181,9 @@ kz_tab_label_class_init (KzTabLabelClass *klass)
 
 	/* GtkWidget signales */
 	widget_class->realize              = kz_tab_label_realize;
+#if 0
 	widget_class->unrealize            = kz_tab_label_unrealize;
+#endif
 	widget_class->button_press_event   = kz_tab_label_button_press;
 	widget_class->scroll_event         = kz_tab_label_scroll_event;
 	widget_class->motion_notify_event  = kz_tab_label_motion_notify;
@@ -208,7 +219,6 @@ kz_tab_label_init (KzTabLabel *kztab)
 	GtkWidget *close_image, *hbox;
 	GtkRcStyle *style;
 	GtkRequisition size;
-	gint i;
 	gint width, height;
 
 	kztab->kz           = NULL;
@@ -221,8 +231,6 @@ kz_tab_label_init (KzTabLabel *kztab)
 	kztab->lock_button  = gtk_image_new_from_stock(KZ_STOCK_ANCHOR,
 						       GTK_ICON_SIZE_MENU);
 	kztab->tooltips     = gtk_tooltips_new();
-	for (i = 0; i < KZ_TAB_LABEL_N_STATE; i++)
-		kztab->style[i] = NULL;
 
 	kztab->priv = g_new0(KzTabLabelPriv, 1);
 	kztab->priv->width             = 80;
@@ -232,7 +240,6 @@ kz_tab_label_init (KzTabLabel *kztab)
 	kztab->priv->lock              = FALSE;
 	kztab->priv->favicon_signal    = 0;
 	
-	kztab->offscreen_pixmap = NULL;
 	kztab->favicon_pixbuf   = NULL;
 	kztab->circle           = NULL;
 	kztab->mask             = NULL;
@@ -344,6 +351,9 @@ kz_tab_label_constructor (GType                  type,
 	g_signal_connect(G_OBJECT(kztab->kzembed), "net-state-all",
 			 G_CALLBACK(cb_net_state_all), kztab);
 
+	/* connect favicon signal */
+	kz_favicon_connect_signal(kztab->kz->kzfav, kztab->kzembed);
+
 	/* set label text */
 	title = kz_moz_embed_ensure_title(kztab->kzembed);
 	kz_tab_label_set_text(kztab, title);
@@ -365,10 +375,6 @@ kz_tab_label_dispose (GObject *object)
 	g_signal_handlers_disconnect_by_func(G_OBJECT(kz_global_profile),
 					     G_CALLBACK(cb_profile_changed),
 					     kztab);
-
-	g_signal_handlers_disconnect_by_func(G_OBJECT(kztab->close_button),
-					     G_CALLBACK(cb_close_button_state_changed),
-					     NULL);
 	if (kztab->kzembed)
 	{
 		g_signal_handlers_disconnect_by_func(G_OBJECT(kztab->kzembed),
@@ -381,10 +387,14 @@ kz_tab_label_dispose (GObject *object)
 						     cb_destroy, kztab);
 		g_signal_handlers_disconnect_by_func(G_OBJECT(kztab->kzembed),
 						     cb_net_state_all, kztab);
+		/* disconnect favicon signal */
+		kz_favicon_disconnect_signal(kztab->kz->kzfav, kztab->kzembed);
+
 		g_object_unref(kztab->kzembed);
 		kztab->kzembed = NULL;
 	}
 
+
 	/* if connected signal exists, disconnect it. */
 	if (kztab->priv->favicon_signal)
 	{
@@ -410,11 +420,6 @@ kz_tab_label_dispose (GObject *object)
 		g_object_unref(kztab->favicon_pixbuf);
 		kztab->favicon_pixbuf = NULL;
 	}
-	if (kztab->offscreen_pixmap)
-	{
-		g_object_unref(kztab->offscreen_pixmap);
-		kztab->offscreen_pixmap = NULL;
-	}
 	if (kztab->circle)
 	{
 		g_object_unref(kztab->circle);
@@ -497,19 +502,28 @@ kz_tab_label_new (KzWindow *kz, KzMozEmbed *kzembed)
 void
 kz_tab_label_set_text(KzTabLabel *kztab, const gchar *text)
 {
+	gchar *escaped, *markup;
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
 	if (!text || !*text) text = _("No title");
-
-	gtk_label_set_text(GTK_LABEL(kztab->label), text);
+	
+	escaped = g_markup_escape_text(text, strlen(text));
+	markup = g_strdup_printf("<span foreground=\"%s\">%s</span>",
+				 label_color[kztab->state],
+				 escaped);
+	gtk_label_set_markup(GTK_LABEL(kztab->label), markup);
 	gtk_tooltips_set_tip(kztab->tooltips, GTK_WIDGET(kztab->eventbox),
 			     text, NULL);
+	g_free(markup);
+	g_free(escaped);
 }
 
 void
 kz_tab_label_set_state (KzTabLabel *kztab,
 			KzTabLabelState state)
 {
+	gchar *title;
+	
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
 	if (!GTK_WIDGET_REALIZED(kztab)) return;
@@ -517,30 +531,20 @@ kz_tab_label_set_state (KzTabLabel *kztab,
 	switch(state)
 	{
 	case KZ_TAB_LABEL_STATE_LOADING:
-		gtk_widget_modify_style(GTK_WIDGET(kztab->label),
-					kztab->style[KZ_TAB_LABEL_STATE_LOADING]);
 		kztab->state = state;
 		break;
-
 	case KZ_TAB_LABEL_STATE_LOADED:
-		gtk_widget_modify_style(GTK_WIDGET(kztab->label),
-					kztab->style[KZ_TAB_LABEL_STATE_LOADED]);
-		gtk_widget_show(kztab->label);
-		gtk_widget_set_size_request(kztab->label, -1, -1);
-		gtk_widget_queue_resize(kztab->label);
 		kztab->state = state;
 		break;
-
 	case KZ_TAB_LABEL_STATE_NORMAL:
 	default:
-		gtk_widget_modify_style(GTK_WIDGET(kztab->label),
-					kztab->style[KZ_TAB_LABEL_STATE_NORMAL]);
-		gtk_widget_show(kztab->label);
-		gtk_widget_set_size_request(kztab->label, -1, -1);
-		gtk_widget_queue_resize(kztab->label);
 		kztab->state = KZ_TAB_LABEL_STATE_NORMAL;
 		break;
 	}
+	
+	title = kz_moz_embed_ensure_title(kztab->kzembed);
+	kz_tab_label_set_text(kztab, title);
+	g_free(title);
 }
 
 KzTabLabelState
@@ -605,77 +609,26 @@ static void
 kz_tab_label_realize (GtkWidget *widget)
 {
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
-	gint i;
 
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
 		GTK_WIDGET_CLASS (parent_class)->realize(widget);
 
 	widget->window = gtk_widget_get_parent_window (widget);
 	g_object_ref (widget->window);
-	widget->style = gtk_style_attach (widget->style, widget->window);
 
 	make_progress_circle(widget);
 
-	/* new rc styles */
-	if (!kztab->style[KZ_TAB_LABEL_STATE_LOADING])
-	{
-		GtkRcStyle *style = gtk_rc_style_new();
-		/* set colors */
-		for(i = 0; i< 5; i++)
-		{
-			style->color_flags[i]   = GTK_RC_FG;
-			style->fg[i].red   = 0xffff;
-			style->fg[i].blue  = 0x0000;
-			style->fg[i].green = 0x0000;
-		}
-		kztab->style[KZ_TAB_LABEL_STATE_LOADING] = style;
-	}
-
-	if (!kztab->style[KZ_TAB_LABEL_STATE_NORMAL])
-	{
-		kztab->style[KZ_TAB_LABEL_STATE_NORMAL]
-			= gtk_widget_get_modifier_style(widget);
-		gtk_rc_style_ref(kztab->style[KZ_TAB_LABEL_STATE_NORMAL]);
-	}
-
-	if (!kztab->style[KZ_TAB_LABEL_STATE_LOADED])
-	{
-		GtkRcStyle *style = gtk_rc_style_new();
-
-		/* set colors */
-		for(i = 0; i< 5; i++)
-		{
-			style->color_flags[i]   = GTK_RC_FG;
-			style->fg[i].red   = 0x2222;
-			style->fg[i].blue  = 0x4444;
-			style->fg[i].green = 0xaaaa;
-		}
-		kztab->style[KZ_TAB_LABEL_STATE_LOADED] = style;
-	}
-
 	if (kz_moz_embed_is_loading(kztab->kzembed))
 		kz_tab_label_set_state (kztab, KZ_TAB_LABEL_STATE_LOADING);
 }
-
+#if 0
 static void
 kz_tab_label_unrealize (GtkWidget *widget)
 {
-	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
-	gint i;
-
-	for (i = 0; i < KZ_TAB_LABEL_N_STATE; i++)
-	{
-		if (kztab->style[i])
-		{
-			gtk_rc_style_unref(kztab->style[i]);
-			kztab->style[i] = NULL;
-		}
-	}
-
 	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
 		return GTK_WIDGET_CLASS (parent_class)->unrealize(widget);
 }
-
+#endif
 static gboolean
 kz_tab_label_button_press (GtkWidget *widget, GdkEventButton *event)
 {
@@ -813,7 +766,7 @@ kz_tab_label_drag_data_get (GtkWidget        *widget,
 	KzTabLabel *kztab;
 	const gchar *uri, *title;
 
-	g_return_if_fail(KZ_TAB_LABEL(widget));
+	g_return_if_fail(KZ_IS_TAB_LABEL(widget));
 	kztab = KZ_TAB_LABEL(widget);
 
 	g_return_if_fail (KZ_IS_MOZ_EMBED(kztab->kzembed));
@@ -889,7 +842,7 @@ kz_tab_label_drag_data_received (GtkWidget *widget,
 {
 	KzTabLabel *kztab;
 
-	g_return_if_fail(KZ_TAB_LABEL(widget));
+	g_return_if_fail(KZ_IS_TAB_LABEL(widget));
 	kztab = KZ_TAB_LABEL(widget);
 
 	g_return_if_fail (KZ_IS_MOZ_EMBED(kztab->kzembed));
@@ -998,8 +951,6 @@ make_progress_circle(GtkWidget *widget)
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
 
 	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-	kztab->offscreen_pixmap = gdk_pixmap_new(widget->window,
-						 width, height, -1);
 	kztab->circle = gdk_pixmap_new(widget->window,
 				       width, height, -1);
 	kztab->mask = gdk_pixmap_new(widget->window,
@@ -1093,16 +1044,10 @@ cb_title_changed (GtkMozEmbed *embed, KzTabLabel *kztab)
 static void
 cb_net_start (GtkMozEmbed *embed, KzTabLabel *kztab)
 {
-	gchar *title;
-
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
 	kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_LOADING);
 
-	title = kz_moz_embed_ensure_title(kztab->kzembed);
-	kz_tab_label_set_text(kztab, title);
-	g_free(title);
-	
 	/* if connected signal exists, disconnect it. */
 	if (kztab->priv->favicon_signal)
 	{
@@ -1117,14 +1062,15 @@ cb_net_start (GtkMozEmbed *embed, KzTabLabel *kztab)
 		g_object_unref(kztab->favicon_pixbuf);
 		kztab->favicon_pixbuf = NULL;
 	}
+	/* show favicon widget for progress maru. */
 	gtk_widget_show(kztab->favicon);
 }
 
 static void
 cb_net_stop (GtkMozEmbed *embed, KzTabLabel *kztab)
 {
-	gchar *title;
 	GtkNotebook *note;
+	gboolean show_favicon;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
@@ -1138,10 +1084,7 @@ cb_net_stop (GtkMozEmbed *embed, KzTabLabel *kztab)
 	{
 		kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_LOADED);
 	}
-	title = kz_moz_embed_ensure_title(kztab->kzembed);
-	kz_tab_label_set_text(kztab, title);
-	g_free(title);
-	
+
 	if (kztab->favicon_pixbuf)
 	{
 		gtk_image_set_from_pixbuf(GTK_IMAGE(kztab->favicon),
@@ -1149,10 +1092,15 @@ cb_net_stop (GtkMozEmbed *embed, KzTabLabel *kztab)
 		gtk_widget_show(kztab->favicon);
 	}
 	else
-		gtk_widget_hide(kztab->favicon);
+		gtk_image_set_from_pixbuf(GTK_IMAGE(kztab->favicon),
+					  NULL);
+
+	KZ_CONF_GET("Tab", "show_favicon", show_favicon, BOOL);
+	kz_tab_label_set_show_favicon(kztab, show_favicon);
 }
 
 
+
 static void
 cb_destroy (GtkObject *object, KzTabLabel *kztab)
 {
@@ -1221,6 +1169,8 @@ cb_net_state_all (GtkMozEmbed *embed, const char *aURI,
 static void
 cb_favicon_changed (KzFavicon *kzfav, gchar *uri, KzTabLabel *kztab)
 {
+	gboolean show_favicon = TRUE;
+	
 	/* if connected signal exists, disconnect it. */
 	if (kztab->priv->favicon_signal)
 	{
@@ -1243,8 +1193,10 @@ cb_favicon_changed (KzFavicon *kzfav, gchar *uri, KzTabLabel *kztab)
 	{
 		gtk_image_set_from_pixbuf(GTK_IMAGE(kztab->favicon),
 					  kztab->favicon_pixbuf);
-		gtk_widget_show(kztab->favicon);
 	}
+
+	KZ_CONF_GET("Tab", "show_favicon", show_favicon, BOOL);
+	kz_tab_label_set_show_favicon(kztab, show_favicon);
 }
 
 
