@@ -24,8 +24,7 @@
 
 #include <gtk/gtk.h>
 
-#include "egg-action-group.h"
-#include "egg-menu-merge.h"
+#include "gtk24backports.h"
 #include "kz-gesture.h"
 #include "kz-embed-event.h"
 #include "kz-favicon.h"
@@ -67,10 +66,10 @@ struct _KzWindow
 	GtkWidget  *dlbox;
 
 	/* other objects */
-	EggActionGroup *actions;
-	EggActionGroup *popup_actions;
-	EggActionGroup *tabpop_actions;
-	EggMenuMerge   *menu_merge;
+	GtkActionGroup *actions;
+	GtkActionGroup *popup_actions;
+	GtkActionGroup *tabpop_actions;
+	GtkUIManager   *menu_merge;
 
 	/* handle for favicon */
 	KzFavicon *kzfav;
@@ -136,21 +135,21 @@ const KzEmbedEventMouse *
 
 #define KZ_WINDOW_SET_SENSITIVE(kz, action_name, sensitive)		       \
 {									       \
-	if (KZ_IS_WINDOW(kz) && EGG_IS_ACTION_GROUP(kz->actions)) {	       \
-		EggAction *action;					       \
-		action = egg_action_group_get_action(kz->actions,	       \
+	if (KZ_IS_WINDOW(kz) && GTK_IS_ACTION_GROUP(kz->actions)) {	       \
+		GtkAction *action;					       \
+		action = gtk_action_group_get_action(kz->actions,	       \
 						     action_name);	       \
-		if (action) egg_action_set_sensitive(action, sensitive);       \
+		if (action) g_object_set(action, "sensitive", sensitive, NULL);\
 	}								       \
 }
 
 #define KZ_WINDOW_SET_VISIBLE(kz, action_name, visible)			       \
 {									       \
-	if (KZ_IS_WINDOW(kz) && EGG_IS_ACTION_GROUP(kz->actions)) {	       \
-		EggAction *action;					       \
-		action = egg_action_group_get_action(kz->actions, 	       \
+	if (KZ_IS_WINDOW(kz) && GTK_IS_ACTION_GROUP(kz->actions)) {	       \
+		GtkAction *action;					       \
+		action = gtk_action_group_get_action(kz->actions, 	       \
 						     action_name);	       \
-		if (action) egg_action_set_visible(action, visible);	       \
+		if (action) g_object_set(action, "visible", visible, NULL);    \
 	}								       \
 }
 
