@@ -24,25 +24,39 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
 
 #include "kazehakase.h"
 #include "kz-icons.h"
 #include "kz-window.h"
+#include "kz-actions-tab.h"
 #include "kz-xbel.h"
 #include "intl.h"
 #include "locale.h"
+#include "utils.h"
 #include "mozilla.h"
 
+#ifdef USE_GTK_2_2
+#include "gtk24backports.h"
+#endif
 
-KzProfile   *kz_global_profile = NULL;
-KzProfile   *kz_session        = NULL;
-KzProfile   *kz_proxy          = NULL;
-KzBookmark  *kz_bookmark       = NULL;
-KzBookmark  *kz_home           = NULL;
-
+KzProfile      *kz_global_profile = NULL;
+KzProfile      *kz_session        = NULL;
+KzProfile      *kz_proxy          = NULL;
+KzRootBookmark *kz_bookmarks      = NULL;
+#if USE_MIGEMO
+Migemo         *migemo            = NULL;
+#endif
+static gchar *usage =
+N_("Usage: kazehakase [OPTION...] [URI...]\n"
+"   -a, --action \"Action Name\"              Execute action\n"
+"   -h, --help                              Show this help\n"
+"   -v, --version                           Show the version\n"
+);
 
+gboolean exists_estindex = FALSE;
 
 /******************************************************************************
  *                                                                            *
@@ -189,6 +203,106 @@ cb_selection_get(GtkWidget *widget,
 }
 
 
+static void
+activate_action(GList *window_list, const gchar *text)
+{
+	KzWindow *kz;
+	KzTabLabel *kztab;
+	GtkWidget *widget;
+	GtkAction *action;
+	gchar *action_name;
+	gint action_type;
+	enum {
+		WINDOW_ACTION,
+		TAB_ACTION
+	};
+	gint window_id = 0, tab_id = -1 /* -1 is current tab */;
+
+	if (!strncmp(text, "KzWindowTabPopup", 16))
+	{
+		gchar *p1, *p2, *p3;
+		action_type = TAB_ACTION;
+		
+		p1 = (gchar *)(text + 17);
+
+		p2 = strchr(p1, ' ');
+		if (!p2)
+		{
+			window_id = 1;
+			action_name = g_strdup(p1);
+		}
+		else
+		{
+			window_id = atoi(p2 + 1);
+			action_name = g_strndup(p1, p2 - p1);
+			p3 = strchr(p2 + 1, ' ');
+			if (p3)
+				tab_id = atoi(p3 + 1);
+		}
+		tab_id = !tab_id ? -1 : tab_id;
+	}
+	else /*if (!strncmp(action, "KzWindow", 8))*/
+	{
+		gchar *p1, *p2;
+		action_type = WINDOW_ACTION;
+
+		p1 = strchr(text, '/');
+		if (!p1)
+			p1 = (gchar*)text;
+		else
+			p1++;
+
+		p2 = strchr(p1, ' ');
+		if (!p2)
+		{
+			window_id = 1;
+			action_name = g_strdup(p1);
+		}
+		else
+		{
+			window_id = atoi(p2 + 1);
+			action_name = g_strndup(p1, p2 - p1);
+		}
+	}
+	window_id = !window_id ? 1 : window_id;
+
+	kz = g_list_nth_data(window_list, window_id - 1);	
+	/* if not exists indicated window, use first one  */
+	if (!kz)
+		kz = window_list->data;
+
+	/* activate actions */
+	switch (action_type)
+	{
+	 case TAB_ACTION:
+		if (tab_id == -1)
+			widget = KZ_WINDOW_CURRENT_PAGE(kz);
+		else
+			widget = KZ_WINDOW_NTH_PAGE(kz, tab_id - 1);
+		if (!widget)
+			widget = KZ_WINDOW_CURRENT_PAGE(kz);
+		if (!widget)
+			break;
+		kztab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook)
+								, widget));
+
+		action = gtk_action_group_get_action(kz->tabpop_actions,
+						     action_name);
+		if (action)
+			kz_actions_tab_activate_action(kztab, action);
+		break;
+	 case WINDOW_ACTION:
+	 default:
+		action = gtk_action_group_get_action(kz->actions,
+						     action_name);
+		if (action)
+			gtk_action_activate(action);
+		break;
+	}
+
+	g_free(action_name);
+}
+
 static void
 cb_selection_received(GtkWidget *widget,
 		      GtkSelectionData *data,
@@ -213,7 +327,22 @@ cb_selection_received(GtkWidget *widget,
 		/* open URL */
 		argv = g_strsplit(data->data, "\n", -1);
 		for (i = 0; argv[i]; i++)
-			kz_window_open_new_tab(kz, argv[i]);
+		{
+			if ((!strcmp(argv[i], "--action") ||
+			     !strcmp(argv[i], "-a")) &&
+			    *argv[i+1] &&
+			    argv[i+1])
+			{
+				activate_action(list, argv[i+1]);
+				i++;
+				continue;
+			}
+
+			gchar *uri = complement_scheme(argv[i]);
+			kz_window_open_new_tab(kz, uri);
+			g_free(uri);
+		}
+
 		g_strfreev(argv);
 	}
 }
@@ -287,9 +416,13 @@ static void
 open_app_info (void)
 {
 	gchar *sysconf_file, *conf_file, *session_file;
-	gchar *bookmark_file;
+	gchar *bookmark_file, *sys_bookmark_file;
+	gchar *scrap_file;
+	gchar *bookmark_bar_file, *sys_bookmark_bar_file;
+	gchar *smartbookmark_file, *sys_smartbookmark_file;
 	gchar *accel_prefs_file;
 	gchar *proxy_file, *sysproxy_file;
+	gchar *estindex;
 
 	/* load prefs */
 	sysconf_file = g_strdup(KZ_SYSCONFDIR "/kzrc");
@@ -303,32 +436,40 @@ open_app_info (void)
 	kz_session = kz_profile_open(session_file, NULL);
 
 	/* load bookmarks */
-	bookmark_file = g_strconcat(g_get_home_dir(),
-				    "/.kazehakase/bookmarks.xml",
-				    NULL);
-	if (g_file_test(bookmark_file, G_FILE_TEST_EXISTS))
-	{
-		kz_bookmark = kz_bookmark_file_new(bookmark_file, _("Bookmarks"),
-						   NULL);
-		kz_bookmark_load_start(kz_bookmark);
-	}
-	else
-		kz_bookmark = kz_bookmark_file_create_new(bookmark_file, _("Bookmarks"),
-							  "XBEL");
+	bookmark_file         = g_strconcat(g_get_home_dir(),
+					    "/.kazehakase/bookmarks.xml",
+					    NULL);
+	sys_bookmark_file     = g_strconcat(KZ_SYSCONFDIR,
+					    "/bookmarks.xml",
+					    NULL);
+
+        bookmark_bar_file     = g_strconcat(g_get_home_dir(),
+					    "/.kazehakase/bookmarkbar.xml",
+					    NULL);
+	sys_bookmark_bar_file = g_strconcat(KZ_SYSCONFDIR,
+					    "/bookmarkbar.xml",
+					    NULL);
+	scrap_file            = g_strconcat(g_get_home_dir(),
+					    "/.kazehakase/scrap.html",
+					    NULL);
+        smartbookmark_file    = g_strconcat(g_get_home_dir(),
+					    "/.kazehakase/smartbookmarks.xml",
+					    NULL);
+	sys_smartbookmark_file= g_strconcat(KZ_SYSCONFDIR,
+					    "/smartbookmarks.xml",
+					    NULL);
+	
+	kz_bookmarks = kz_root_bookmark_new(bookmark_file, sys_bookmark_file,
+					    scrap_file, NULL);
+
+	kz_root_bookmark_add_smartbookmark_file(kz_bookmarks,
+						smartbookmark_file,
+						sys_smartbookmark_file);
+	kz_root_bookmark_add_bookmark_bar_file(kz_bookmarks,
+					       bookmark_bar_file,
+					       sys_bookmark_bar_file);
+
 
-	bookmark_file = g_strconcat(g_get_home_dir(),
-				    "/.kazehakase/home.xml",
-				    NULL);
-	if (g_file_test(bookmark_file, G_FILE_TEST_EXISTS))
-	{
-		kz_home = kz_bookmark_file_new(bookmark_file, _("Home & Lairs"),
-					       NULL);
-		kz_bookmark_load_start(kz_home);
-	}
-	else
-		kz_home = kz_bookmark_file_create_new(bookmark_file, _("Home & Lairs"),
-						      "XBEL");
-		
 	/* Load other prefs... */
 	accel_prefs_file = g_strconcat (g_get_home_dir(),
 					"/.kazehakase/keyaccelrc",
@@ -342,10 +483,23 @@ open_app_info (void)
 	kz_icons_init();
 	mozilla_init();
 
+	/* check existence of estindex */
+	estindex = g_find_program_in_path("estindex");
+	
+	if (estindex)
+	{
+		exists_estindex = TRUE;
+		g_free(estindex);
+	}
 	g_free(sysconf_file);
 	g_free(conf_file);
 	g_free(session_file);
 	g_free(bookmark_file);
+	g_free(scrap_file);
+	g_free(bookmark_bar_file);
+	g_free(sys_bookmark_bar_file);
+	g_free(smartbookmark_file);
+	g_free(sys_smartbookmark_file);
 	g_free(accel_prefs_file);
 	g_free(sysproxy_file);
 	g_free(proxy_file);
@@ -366,19 +520,24 @@ close_app_info (void)
 
 	mozilla_exit();
 
+	kz_root_bookmark_save_all(kz_bookmarks);
+	g_object_unref(G_OBJECT(kz_bookmarks));
+
 	kz_profile_close(kz_global_profile);
 	kz_profile_close(kz_session);
 	kz_profile_close(kz_proxy);
 
-	kz_bookmark_save(kz_home);
-	g_object_unref(G_OBJECT(kz_home));
-	kz_bookmark_save(kz_bookmark);
-	g_object_unref(G_OBJECT(kz_bookmark));
-
+#if USE_MIGEMO
+	if (migemo)
+	{
+		migemo_exit();
+		migemo = NULL;
+	}
+#endif	
 	kz_global_profile = NULL;
 	kz_session        = NULL;
 	kz_proxy          = NULL;
-	kz_bookmark       = NULL;
+	kz_bookmarks      = NULL;
 }
 
 
@@ -387,7 +546,7 @@ main (int argc, char *argv[])
 {
 	KzWindow *kz = NULL;
 	GtkWidget *dupl_check_server = NULL;
-	const gchar *url = NULL;
+	gchar *uri = NULL;
 	gint i;
 
 	/* initialize */
@@ -396,9 +555,27 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
 
-	gtk_set_locale();
 	gtk_init(&argc, &argv);
 
+	for (i = 1; i < argc; i++)	
+	{
+		if (!strncmp(argv[i], "--version", 9) ||
+		    !strncmp(argv[i], "-v", 2))
+		{
+			g_print("%s %s\n", PACKAGE, VERSION);
+			return 0;
+		}
+		else if (!strncmp(argv[i], "--help", 6)||
+			 !strncmp(argv[i], "-h", 2))
+		{
+			g_print(_(usage));
+			return 0;
+		}
+	}
+	
+#ifdef USE_GTK_2_2
+	gtk24backports_init();
+#endif
 	/* find kazehakase process */
 	dupl_check_server = check_kazehakase_process(argc, argv);
 	if (!dupl_check_server)
@@ -410,12 +587,19 @@ main (int argc, char *argv[])
 	/* Load preference */
 	open_app_info();
 
-	/* create window */
+	/* create window */	
 	if (argc > 1)
-		url = argv[1];
-	kz = KZ_WINDOW(kz_window_new(url));
+		uri = complement_scheme(argv[1]);
+	kz = KZ_WINDOW(kz_window_new(uri));
+	if (uri)
+		g_free(uri);
+
 	for (i = 2; i < argc; i++)
-		kz_window_open_new_tab(kz, argv[i]);
+	{
+		uri = complement_scheme(argv[i]);
+		kz_window_open_new_tab(kz, uri);
+		g_free(uri);
+	}
 	gtk_widget_show(GTK_WIDGET(kz));
 
 	/* event loop */
