@@ -24,7 +24,6 @@
 #include <string.h>
 #include <gtkmozembed.h>
 #include <gtkmozembed_internal.h>
-#include <regex.h>
 
 #include "kazehakase.h"
 #include "kz-mozwrapper.h"
@@ -34,49 +33,53 @@
 #include "mozilla.h"
 #include "mozilla-prefs.h"
 #include "bookmarks/kz-bookmark.h"
-#if 0
+#if USE_MIGEMO
 #include "migemo.h"
 #endif
+#include "utils.h"
 
-#include <xpcom/nsCOMPtr.h>
-#include <dom/nsIDOMDocument.h>
+#include <nsCOMPtr.h>
+#include <nsIDOMDocument.h>
 #include <content/nsIDocumentViewer.h>
 #include <content/nsIContent.h>
-#include <webbrwsr/nsIWebBrowser.h>
+#include <nsIWebBrowser.h>
 #include <content/nsIDocument.h>
-#include <dom/nsIDOMMouseEvent.h>
+#include <nsIDOMMouseEvent.h>
 #include <dom/nsIDOMKeyEvent.h>
-#include <dom/nsIDOMEventTarget.h>
+#include <nsIDOMEventTarget.h>
 #include <dom/nsIDOMNSHTMLElement.h>  
-#include <dom/nsIDOMHTMLElement.h>  
-#include <dom/nsIDOMHTMLTextAreaElement.h>  
-#include <dom/nsIDOMNamedNodeMap.h>
+#include <nsIDOMHTMLElement.h>  
+#include <nsIDOMHTMLTextAreaElement.h>  
+#include <nsIDOMNamedNodeMap.h>
 #include <webbrowserpersist/nsIWebBrowserPersist.h>
 #include <necko/nsNetUtil.h>
-#include <find/nsIWebBrowserFind.h>
+#include <nsIWebBrowserFind.h>
 #include <dom/nsIDOMNSDocument.h>
 #include <dom/nsIDOMNSEvent.h>
 #include <docshell/nsIDocShell.h>
 #include <docshell/nsIDocShellTreeItem.h>
 #include <docshell/nsIDocShellTreeOwner.h>
-#include <dom/nsIDOMNodeList.h>
-#include <dom/nsIDOMWindow.h>
-#include <content/nsISelection.h>
-#include <dom/nsIDOMRange.h>
-#include <find/nsIWebBrowserFind.h>
+#include <nsIDOMNodeList.h>
+#include <nsIDOMWindow.h>
+#include <nsISelection.h>
+#include <nsIDOMRange.h>
+#include <nsIWebBrowserFind.h>
 #include <necko/nsNetUtil.h>
 #include <uconv/nsICharsetConverterManager.h>
 #if MOZILLA_SNAPSHOT < 10
 #	include <uconv/nsICharsetConverterManager2.h>
 #endif
-#include <dom/nsIDOMWindow.h>
-#include <content/nsISelection.h>
+#include <nsIDOMWindow.h>
+#include <nsISelection.h>
 #include <nsISHistory.h>
 #include <nsIHistoryEntry.h>
 #include <nsISHEntry.h>
 #include <nsIWebNavigation.h>
 #include <nsCWebBrowserPersist.h>
 #include <widget/nsIBaseWindow.h>
+#include <nsIWebPageDescriptor.h>
+#include <nsIPresContext.h>
+#include <nsIEventStateManager.h>
 
 struct _KzMozEmbedPriv
 {
@@ -96,7 +99,7 @@ struct _KzMozEmbedPriv
 	
 	/* for alternate rss */
 	gchar *link_rss;
-#if 0	
+#if USE_MIGEMO
 	/* for migemo */
 	gchar *migemo_keyword;
 #endif
@@ -263,10 +266,9 @@ kz_moz_embed_init (KzMozEmbed *kzembed)
 	kzembed->priv->link_next      = NULL;
 	kzembed->priv->link_prev      = NULL;
 	kzembed->priv->link_rss       = NULL;
-#if 0
+#if USE_MIGEMO
 	kzembed->priv->migemo_keyword = NULL;
 #endif
-	kz_moz_embed_load_url(kzembed, "about:blank");
 }
 
 GtkWidget *
@@ -297,7 +299,7 @@ kz_moz_embed_destroy (GtkObject *object)
 	kzembed->location = NULL;
 	g_free(kzembed->title);
 	kzembed->title = NULL;
-#if 0	
+#if USE_MIGEMO
 	if (kzembed->priv->migemo_keyword)
 	{
 		g_free(kzembed->priv->migemo_keyword);
@@ -448,15 +450,15 @@ kz_moz_embed_load_url (KzMozEmbed *kzembed, const gchar *url)
 void
 kz_moz_embed_view_source (KzMozEmbed *kzembed, const gchar *url)
 {
-	gchar *source;
-	
-	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
+	KzMozWrapper *wrapper = KZ_MOZ_EMBED(kzembed)->priv->wrapper;
+	nsresult rv;
 
-	source = g_strdup_printf("view-source:%s", url);
-	
-	kz_moz_embed_load_url(kzembed, source);
-	
-	g_free(source);
+        nsCOMPtr<nsISupports> pageDescriptor;
+        rv = wrapper->GetPageDescriptor(getter_AddRefs(pageDescriptor));
+        if (!pageDescriptor || NS_FAILED(rv)) return;
+
+        rv = wrapper->LoadDocument(pageDescriptor, 
+				   nsIWebPageDescriptor::DISPLAY_AS_SOURCE);
 }
 
 
@@ -646,7 +648,6 @@ kz_moz_embed_net_state_all (GtkMozEmbed *embed, const char *aURI,
                 else if (state & GTK_MOZ_EMBED_FLAG_STOP)
                         kzembed->priv->cur_requests ++;
         }
-
 	if (parent_class->net_state_all)
 		parent_class->net_state_all(embed, aURI, state, status);
 }
@@ -846,13 +847,14 @@ kz_moz_embed_find (KzMozEmbed *embed, const char *keyword,
 
 	nsresult rv;
 	nsCOMPtr<nsIWebBrowserFind> finder(do_GetInterface(web));
-#if 0
+#if USE_MIGEMO
 	if (migemo && embed->priv->migemo_keyword)
 		finder->SetSearchString(NS_ConvertUTF8toUCS2(embed->priv->migemo_keyword).get());
 	else
 		finder->SetSearchString(NS_ConvertUTF8toUCS2(keyword).get());
-#endif
+#else
 	finder->SetSearchString(NS_ConvertUTF8toUCS2(keyword).get());
+#endif
 	finder->SetFindBackwards(backward);
 	finder->SetWrapFind(TRUE);
 	finder->SetEntireWord(TRUE);
@@ -889,7 +891,7 @@ kz_moz_embed_incremental_search (KzMozEmbed *embed, const char *keyword,
 	}
 
 	nsCOMPtr<nsIWebBrowserFind> finder(do_GetInterface(web));
-#if 0
+#if USE_MIGEMO
 	gboolean use_migemo;
 	KZ_CONF_GET("Global", "use_migemo", use_migemo, BOOL);
 
@@ -910,30 +912,15 @@ kz_moz_embed_incremental_search (KzMozEmbed *embed, const char *keyword,
 			}
 		}
 
-		gchar *regex = migemo_get_regex(keyword);		
-		
-		if (regex)
-		{
-			regex_t preg;
-			regmatch_t mat;
+		gchar *word = migemo_get_word(body_string, keyword);
 
-			int ret = regcomp(&preg, regex, REG_EXTENDED);
+		if (word) {
+			if (embed->priv->migemo_keyword)
+				g_free(embed->priv->migemo_keyword);
 
-			if (ret == 0)
-			{
-				ret = regexec(&preg, body_string, 1, &mat, 0);
-				if (ret == 0)
-				{
-					if (embed->priv->migemo_keyword)
-						g_free(embed->priv->migemo_keyword);
-					embed->priv->migemo_keyword = 
-						g_strndup(body_string + mat.rm_so,
-							  mat.rm_eo - mat.rm_so); 
-					finder->SetSearchString(NS_ConvertUTF8toUCS2(embed->priv->migemo_keyword).get());
-				}
-				regfree(&preg);
-			}
-			g_free(regex);
+			embed->priv->migemo_keyword = word;
+
+			finder->SetSearchString(NS_ConvertUTF8toUCS2(embed->priv->migemo_keyword).get());
 		}
 		g_free(body_string);
 	}
@@ -941,9 +928,10 @@ kz_moz_embed_incremental_search (KzMozEmbed *embed, const char *keyword,
 	{
 		finder->SetSearchString(NS_ConvertUTF8toUCS2(keyword).get());
 	}
-#endif	
+#else
 	finder->SetSearchString(NS_ConvertUTF8toUCS2(keyword).get());
-START_SEARCH:	
+#endif
+START_SEARCH:
 	finder->SetFindBackwards(backward);
 	finder->SetWrapFind(TRUE);
 	finder->SetEntireWord(TRUE);
@@ -1040,7 +1028,7 @@ kz_moz_embed_get_links (KzMozEmbed *kzembed, GList **list,
 
 
 void
-kz_moz_embed_add_ui (KzMozEmbed *kzembed, EggMenuMerge *ui, KzUILevel level)
+kz_moz_embed_add_ui (KzMozEmbed *kzembed, GtkUIManager *ui, KzUILevel level)
 {
 	g_warning("kz_moz_embed_add_ui() is not implemented yet");
 }
@@ -1994,3 +1982,72 @@ kz_moz_embed_set_text_into_textarea (KzMozEmbed *kzembed,
 	
 	return TRUE;
 }
+
+
+gchar *
+kz_moz_embed_store_history_file(KzMozEmbed *kzembed)
+{
+	nsresult rv;
+	KzMozWrapper *wrapper = NULL;
+	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
+
+	wrapper = kzembed->priv->wrapper;
+	g_return_val_if_fail (wrapper != NULL, NULL);
+
+	PRUint32 persistFlags = 0;
+	
+	nsCOMPtr<nsIWebBrowserPersist> bpersist = 
+		do_CreateInstance(NS_WEBBROWSERPERSIST_CONTRACTID);
+	if (!bpersist) return NULL;
+
+	persistFlags = nsIWebBrowserPersist::PERSIST_FLAGS_REPLACE_EXISTING_FILES;
+
+	persistFlags |= nsIWebBrowserPersist::PERSIST_FLAGS_FROM_CACHE;
+	bpersist->SetPersistFlags (persistFlags);
+
+	nsCOMPtr<nsIURI> inURI;
+	nsCAutoString sURI;
+
+	rv = wrapper->GetDocumentUrl(sURI);
+      	rv = NS_NewURI(getter_AddRefs(inURI), sURI);
+
+	gchar *dir, *filename;
+	gchar *uri = ToNewCString(sURI);
+
+	filename = create_filename_with_path_from_uri(uri);
+		
+	if (!filename)
+		filename = g_strdup("index.html");
+
+	dir = g_strconcat(g_get_home_dir(),
+			  "/.kazehakase/history/",
+			  filename,
+			  NULL);
+	g_free(filename);
+	g_free(uri);
+
+	nsCOMPtr<nsILocalFile> localFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
+	rv = localFile->InitWithPath(NS_ConvertUTF8toUCS2(dir));
+
+	if (NS_FAILED(rv))
+		return NULL;
+
+	PRBool exists;
+	localFile->Exists(&exists);
+	if (!exists)
+	{
+		rv = localFile->Create(nsIFile::NORMAL_FILE_TYPE, 0600);
+		if (NS_FAILED(rv))
+			return NULL;
+	}
+
+	nsCOMPtr<nsISupports> pageDescriptor;
+
+	wrapper->GetPageDescriptor(getter_AddRefs(pageDescriptor));
+	
+	rv = bpersist->SaveURI (inURI, pageDescriptor, nsnull,
+				nsnull, nsnull, localFile);
+	
+	return dir;
+}
+
