@@ -42,6 +42,10 @@ enum {
 	PROP_PIXBUF
 };
 
+enum {
+	ICON_PRESSED_SIGNAL,
+	LAST_SIGNAL
+};
 
 /* object class */
 static void kz_entry_class_init     (KzEntryClass   *klass);
@@ -79,6 +83,8 @@ static void     get_widget_window_size             (GtkEntry       *entry,
 						    gint           *y,
 						    gint           *width,
 						    gint           *height);
+static gint kz_entry_signals[LAST_SIGNAL] = {0};
+
 static GtkEntryClass *parent_class = NULL;
 
  
@@ -109,6 +115,18 @@ kz_entry_class_init (KzEntryClass *klass)
 	widget_class->button_release_event = kz_entry_button_release;
 	widget_class->size_allocate        = kz_entry_size_allocate;
 
+	klass->icon_pressed = NULL;
+
+	kz_entry_signals[ICON_PRESSED_SIGNAL]
+		= g_signal_new ("icon-pressed",
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+				G_STRUCT_OFFSET (KzEntryClass, icon_pressed),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__OBJECT,
+				G_TYPE_NONE, 1,
+				GDK_TYPE_EVENT);
+	
 	g_object_class_install_property (gobject_class,
 					 PROP_BACKTEXT,
 					 g_param_spec_string ("text",
@@ -267,6 +285,7 @@ kz_entry_realize (GtkWidget *widget)
 
 	attributes.x -= KZ_ENTRY(entry)->icon_width + INNER_BORDER;
 	attributes.width = KZ_ENTRY(entry)->icon_width + INNER_BORDER;
+	attributes.cursor = gdk_cursor_new(GDK_LEFT_PTR);
 	KZ_ENTRY(entry)->icon_area = gdk_window_new (widget->window, &attributes, attributes_mask);
 	gdk_window_set_user_data (KZ_ENTRY(entry)->icon_area, entry);
 
@@ -1229,18 +1248,30 @@ kz_entry_expose (GtkWidget *widget,
 	else if (KZ_ENTRY(entry)->icon_area == event->window)
 	{
 		GdkGC *gc;
+		gint x, y; 
+		GdkPixbuf *scale_pixbuf;
+		get_text_area_size (GTK_ENTRY(entry),
+				    NULL, NULL,
+				    &x, &y);
+		y = y - INNER_BORDER * 2;
+		x = y;	
 		switch (entry->icon_type)
 		{
 		 case KZ_ENTRY_ICON_PIXBUF:
 			gc = gdk_gc_new(KZ_ENTRY(entry)->icon_area);
+			scale_pixbuf = gdk_pixbuf_scale_simple(entry->pixbuf,
+						 x, 
+						 y,
+						 GDK_INTERP_NEAREST);
 			gdk_draw_pixbuf(KZ_ENTRY(entry)->icon_area,
 					gc,
-					entry->pixbuf,
+					scale_pixbuf,
 					0, 0,
 					INNER_BORDER, INNER_BORDER,
 					-1, -1,
 					GDK_RGB_DITHER_NONE,
 					0, 0);
+			g_object_unref(scale_pixbuf);
 			g_object_unref(gc);
 			break;
 		 case KZ_ENTRY_ICON_STOCK:
@@ -1257,9 +1288,13 @@ kz_entry_expose (GtkWidget *widget,
 							   entry->icon_size,
 							   widget,
 							   "entry icon");
+			scale_pixbuf = gdk_pixbuf_scale_simple(pixbuf,
+						 x, 
+						 y,
+						 GDK_INTERP_NEAREST);
 			gdk_draw_pixbuf(KZ_ENTRY(entry)->icon_area,
 					gc,
-					pixbuf,
+					scale_pixbuf,
 					0, 0,
 					INNER_BORDER, INNER_BORDER,
 					-1, -1,
@@ -1267,6 +1302,7 @@ kz_entry_expose (GtkWidget *widget,
 					0, 0);
 			g_object_unref(gc);
 			g_object_unref(pixbuf);
+			g_object_unref(scale_pixbuf);
 			break;
 			}
 		 default:
@@ -1442,16 +1478,24 @@ static gboolean
 kz_entry_button_press (GtkWidget *widget,
 		       GdkEventButton *event)
 {
+	KzEntry *entry = KZ_ENTRY(widget);
 	gboolean ret = FALSE;
 
+	if (event->window == entry->icon_area)
+	{
+		g_signal_emit(G_OBJECT(widget), 
+			      kz_entry_signals[ICON_PRESSED_SIGNAL],
+			      0, event);
+	}
+
 	if (!GTK_WIDGET_HAS_FOCUS (widget))
-		KZ_ENTRY(widget)->from_outside = TRUE;
+		entry->from_outside = TRUE;
 	else
-		KZ_ENTRY(widget)->from_outside = FALSE;
+		entry->from_outside = FALSE;
 
 	if (GTK_WIDGET_CLASS(parent_class)->button_press_event)
 		ret = GTK_WIDGET_CLASS(parent_class)->button_press_event(widget, event);
-	
+
 	return ret;
 }
 
